/* $Id: ThreadIDRegistryImpl.java,v 1.1 2002/06/26 20:56:02 hnetynka Exp $ */
package SOFA.Component;

/** Implementation of the thread registry. 
  * 
  */
public class ThreadIDRegistryImpl implements ThreadIDRegistry {
  protected java.util.Hashtable threadIds;
  protected java.util.Hashtable insideThreads;

  /// SOFAnode name
  protected String nodeName;  

  /// lock for testing  inUpdate  variable
  private SOFA.Util.Lock lock;
  private boolean inUpdate;

  // wait while no thread is in component
  private Waiting updateWt;
  // incomming threads wait for update end
  private Waiting thrWt;
  
  

  public ThreadIDRegistryImpl(String _nodeName) {
    threadIds = new java.util.Hashtable();
    insideThreads = new java.util.Hashtable();
    nodeName = _nodeName;
    lock = new SOFA.Util.Lock(false);
    inUpdate = false;
    updateWt = new Waiting();
    thrWt    = new Waiting();
  }

  public void setInUpdate(boolean upd) {
    if (upd) {
      lock.lock();
      inUpdate = true;
      lock.unlock();
      while (! insideThreads.isEmpty()) {
        try {
          updateWt.waitForSignal();
        } catch (InterruptedException e) { }
      }
    } else { //unset inUpdate
      lock.lock();
      inUpdate = false;
      lock.unlock();
      thrWt.signalAll();
    }
  }

  public String threadInCall(java.lang.String thr_id) {
    lock.lock();
    while (inUpdate) {
      lock.unlock();
      try {
        thrWt.waitForSignal();
      } catch (InterruptedException e) {}
      lock.lock();
    }
    lock.unlock();
    if (thr_id == null)
      thr_id = generateThreadId();
    threadIds.put(Thread.currentThread(), thr_id);
    if (! insideThreads.containsKey(thr_id)) {
      insideThreads.put(thr_id, new Counter());
    } else {
      ((Counter) insideThreads.get(thr_id)).inc();
    }
    return thr_id;
  }

  public String threadOutReturn(java.lang.String thr_id) {
    if (((Counter)insideThreads.get(thr_id)).dec()==0) {
      insideThreads.remove(thr_id);
    }
    updateWt.signal();
    return thr_id;
  }

  public String threadOutCall() {
    String id = (String) threadIds.get(Thread.currentThread());
    if (id == null) {
      id = generateThreadId();
      threadIds.put(Thread.currentThread(), id);
    }
    return id;
  }

  public String threadInReturn(String thr_id) {
    ;  // do nothing
    return thr_id;
  }

  /** Generates globaly unique thread identifier.
    * @return   thread identifier
    */
  private String generateThreadId() {
    String ip = nodeName;
    ip = ip + ":" + Long.toString(System.currentTimeMillis());
    ip = ip + ":" + Thread.currentThread().getName();
    return ip;
  }
}

class Counter {
  private long num;
  
  public Counter() {
    num = 1;
  }
  
  public long inc() {
    return ++num;
  }
  
  public long dec() {
    if (num != 0)
      num--;
    return num;
  }
  
  public long get() {
    return num;
  }
}

class Waiting {
  public synchronized void waitForSignal() throws InterruptedException {
    wait();
  }

  public synchronized void signal() {
    notify();
  }

  public synchronized void signalAll() {
    notifyAll();
  }
}
