/*
 * ECGenerator.java
 *
 * Created on 4. duben 2002, 11:22
 */

package SOFA.Connector.ECG;

import org.w3c.dom.*;

/**
 *
 * @author  ghort
 * @version
 */
public class ECGenerator {
	
	protected java.util.LinkedList plugin; /* of ECGPluginInterface */
	
	protected javax.xml.parsers.DocumentBuilderFactory documentBuilderFactory;
	protected javax.xml.parsers.DocumentBuilder documentBuilder;
	
	protected Document configDoc;

	protected SOFA.Connector.EEG.EEGenerator eeGenerator;

	static public String getTypesPackage() {
		return "SOFA.Connector.ECG.Types";
	}
		
	/** Creates new EEGenerator */
	public ECGenerator() throws ECGeneratorException {
		try {
			eeGenerator=new SOFA.Connector.EEG.EEGenerator();	
		} catch (SOFA.Connector.EEG.EEGeneratorException e) {
			throw new ECGeneratorException("Can't initialize EEGenerator.",e);
		}
		
		try {
			documentBuilderFactory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
			documentBuilder=documentBuilderFactory.newDocumentBuilder();
			
			configDoc=documentBuilder.parse(new java.io.File(getECGConfigRoot()+"/"+System.getProperty("SOFA.Connector.ECG.Config","conf.xml")));
			
			Element conParent=configDoc.getDocumentElement();
			Node conPlugins, conPlugin, conProp;
			
			plugin=new java.util.LinkedList();
			
			System.out.println("Loading ECG plugins:");
			conPlugins=conParent.getFirstChild();
			while (conPlugins!=null) {
				if (conPlugins.getNodeType()==Node.ELEMENT_NODE && conPlugins.getNodeName().equals("ECGPlugins")) {
					conPlugin=conPlugins.getFirstChild();
					while (conPlugin!=null) {
						if (conPlugin.getNodeType()==Node.ELEMENT_NODE && conPlugin.getNodeName().equals("ECGPlugin")) {
							
							java.util.HashMap props=new java.util.HashMap();
							conProp=conPlugin.getFirstChild();
							while (conProp!=null) {
								if (conProp.getNodeType()==Node.ELEMENT_NODE) {
									Element el=(Element)conProp;
									if (el.getTagName().equals("property") && el.hasAttribute("name") && el.hasAttribute("value")) {
										props.put(el.getAttribute("name"),el.getAttribute("value"));
									}
								}
								conProp=conProp.getNextSibling();
							}
							System.out.println("  Loading plugin ("+((Element)conPlugin).getAttribute("class")+"):");
							
							Class cls=Class.forName(((Element)conPlugin).getAttribute("class"));
							Class[] coPar=new Class[2];
							coPar[0]=Class.forName("SOFA.Connector.EEG.EEGenerator");
							coPar[1]=Class.forName("java.util.HashMap");
							java.lang.reflect.Constructor co=cls.getConstructor(coPar);
							
							Object[] coInst=new Object[2];
							coInst[0]=eeGenerator;
							coInst[1]=props;
							ECGPluginInterface plgObject=(ECGPluginInterface)co.newInstance(coInst);
							
							plugin.add(plgObject);
							System.out.println("  Plugin "+plgObject.getName()+" ("+((Element)conPlugin).getAttribute("class")+") loaded.");
						}
						conPlugin=conPlugin.getNextSibling();
					}
				}
				conPlugins=conPlugins.getNextSibling();
			}
			
		} catch (Exception e) {
			throw new ECGeneratorException("Can't parse configuration",e);
		}
	}
	
	public ConnectorsTechnologyDescriptor canGenerate(ConnectorQuery query) {
		java.util.Iterator iter;
		ECGPluginInterface plgObject;
		ConnectorsTechnologyDescriptor ctDescr=new ConnectorsTechnologyDescriptor();
		java.util.LinkedList tDescr=new java.util.LinkedList();
		
		ctDescr.type=query.type;
		ctDescr.instanceName=query.instanceName;
		iter=plugin.iterator();
		try {
			for (;;) {
				plgObject=(ECGPluginInterface)iter.next();
				tDescr.addAll(java.util.Arrays.asList(plgObject.canGenerate(query)));
			}
		} catch (java.util.NoSuchElementException e) {
		}

		int i;
		Object arr[];
		ctDescr.technologies=new ConnectorTechnologyDescriptor[(arr=tDescr.toArray()).length];
		for (i=0;i<arr.length;i++) {
			ctDescr.technologies[i]=(ConnectorTechnologyDescriptor)arr[i];
		}

		return ctDescr;
	}
	
	public ConnectorOutputDescriptor generate(ConnectorInputDescriptor conDescr) throws ECGeneratorException {
		java.util.Iterator iter;
		ECGPluginInterface plgObject;
		
		int colonIndex=conDescr.implementation.indexOf(':');
		String pluginName;
		
		if (colonIndex==-1) {
			pluginName=conDescr.implementation;
		} else {
			pluginName=conDescr.implementation.substring(0,colonIndex);
		}
		
		System.out.println("Generating connector '"+conDescr.implementation+"':");

		iter=plugin.iterator();
		try {
			for (;;) {
				plgObject=(ECGPluginInterface)iter.next();
				if (pluginName.equals(plgObject.getName())) {
					try {
						ConnectorOutputDescriptor ret;
						ret=plgObject.generate(conDescr);
						System.out.println("Connector generated.");
						return ret;
					} catch (ECGPluginException e) {
						throw new ECGeneratorException("Plugin failed",e);
					}
				}
			}
		} catch (java.util.NoSuchElementException e) {
			throw new ECGeneratorException("Can't generate element.",e);
		}
	}

	static public String getECGRoot() throws ECGeneratorException {
		try {
			return System.getProperty("SOFA.Connector.Root","SOFA/Connector")+"/ECG";
		} catch (Exception e) {
			throw new ECGeneratorException("Can't access property SOFA.Connector.Root",e);
		}
	}

	static public String getECGConfigRoot() throws ECGeneratorException {
		String configDir;
		try {
			configDir=System.getProperty("SOFA.Connector.Config");
		} catch (Exception e) {
			throw new ECGeneratorException("Can't access property SOFA.Connector.Config",e);
		}
		if (configDir==null) {
			configDir=getECGRoot();
		} else {
			configDir+="/ECG";
		}
		return configDir;
	}
}
