/*
 * JWriter.java
 *
 * Created on 7. duben 2002, 10:23
 */

package SOFA.Connector.EEG.CodeWriter;

import SOFA.SOFAnode.Made.TIR.*;
import SOFA.SOFAnode.Made.TIR.Access.TIRAccessMethods;

/**
 *
 * @author  ghort
 * @version 
 */
public class JWriter {

	protected InterfaceDef inIface;
	
	protected String outFile;
	protected String outClass;
	protected String outPackage;

	/** Creates new JWriter */
	public JWriter(String iface) throws CodeWriterException {
	 	try {
			inIface=(InterfaceDef)TIRAccessMethods.lookupCDLContained(SOFA.Connector.TIRAccess.TIRAccess.repository,iface);
		} catch (Exception e) {
			throw new CodeWriterException("Can't access interface '"+iface+"'.",e);
		}
	}

	public JWriter() {
	};

	public void setOut(String outFile, String outPackage, String outClass) {
		this.outFile=outFile;
		this.outClass=outClass;
		this.outPackage=outPackage;
	}

	public void write(JWriterDirector director, String implIface) throws CodeWriterException {
		java.io.PrintWriter out;
		try {
			out=new java.io.PrintWriter(new java.io.BufferedWriter(new java.io.FileWriter(outFile)));
		} catch (Exception e) {
			throw new CodeWriterException("Can't open output file.",e);
		}

		try {
			int i;
			Contained[] mts=TIRAccessMethods.getOperAttrOfInterface(inIface);

			out.println("package "+outPackage+";");
			out.println();
			out.print("public class "+outClass);
			
			String extnds=director.getExtends();
			if (extnds!=null) {
				out.print(" extends "+extnds);
			}
			
			String[] impls=director.getImplements();
			if (director.defaultImplements()) {
				out.print(" implements "+implIface);
			} else {
				if (impls!=null) {
					out.print(" implements ");
				}
			}
			if (impls!=null) {
				for (i=0;i<impls.length;i++) {
					if (director.defaultImplements() || i>0) {
						out.print(", ");
					}
					out.print(impls[i]);
				}
			}
			
			out.println(" {");
			out.println();

			String clsBody=director.getClassBody();
			if (clsBody!=null) {
				out.println(clsBody);
			}
			
			for (i=0;i<mts.length;i++) {
				OperationDef mt=(OperationDef)mts[i];
				String methodDef=director.getMethodDef(mt);
				
				if (methodDef!=null) {
					out.println("\tpublic "+methodDef+" {");
					out.println(director.getMethodBody(mt));
					out.println("\t}\n");
				}
			}
			
			out.println("}");
		} catch (JWriterDirectorException e) {
			throw new CodeWriterException("Troubles with java writer director.",e);
		} catch (java.rmi.RemoteException e) {
			throw new CodeWriterException("Can't access TIR.",e);
		} catch (SOFA.SOFAnode.Made.TIR.TIRExceptLock e) {
			throw new CodeWriterException("TIR is locked.",e);
		}

		out.close();
	}

	public void write(JWriterDirector director) throws CodeWriterException {
		java.io.PrintWriter out;
		try {
			out=new java.io.PrintWriter(new java.io.BufferedWriter(new java.io.FileWriter(outFile)));
		} catch (Exception e) {
			throw new CodeWriterException("Can't open output file.",e);
		}

		try {
			int i;

			out.println("package "+outPackage+";");
			out.println();
			out.print("public class "+outClass);
			
			String extnds=director.getExtends();
			if (extnds!=null) {
				out.print(" extends "+extnds);
			}
			
			String[] impls=director.getImplements();
			if (impls!=null) {
				out.print(" implements ");

				for (i=0;i<impls.length;i++) {
					if (director.defaultImplements() || i>0) {
						out.print(", ");
					}
					out.print(impls[i]);
				}
			}
			
			out.println(" {");
			out.println();

			String clsBody=director.getClassBody();
			if (clsBody!=null) {
				out.println(clsBody);
			}
			
			out.println("}");
		} catch (JWriterDirectorException e) {
			throw new CodeWriterException("Troubles with java writer director.",e);
		}

		out.close();
	}
	
	public void writeInterface(JWriterInterfaceDirector director) throws CodeWriterException {
		java.io.PrintWriter out;
		try {
			out=new java.io.PrintWriter(new java.io.BufferedWriter(new java.io.FileWriter(outFile)));
		} catch (Exception e) {
			throw new CodeWriterException("Can't open output file.",e);
		}

		try {
			int i;
			Contained[] mts=TIRAccessMethods.getOperAttrOfInterface(inIface);

			out.println("package "+outPackage+";");
			out.println();
			out.print("public interface "+outClass);
			
			String[] exts=director.getExtends();
			if (exts!=null) {
				out.print(" extends ");
				for (i=0;i<exts.length;i++) {
					if (i>0) {
						out.print(", ");
					}
					out.print(exts[i]);
				}
			}
			
			out.println(" {");
			out.println();

			String ifaceBody=director.getInterfaceBody();
			if (ifaceBody!=null) {
				out.println(ifaceBody);
			}
			
			for (i=0;i<mts.length;i++) {
				String methodDef=director.getMethod((OperationDef)mts[i]);
				
				if (methodDef!=null) {
					out.println("\t"+methodDef+";");
				}
			}
			
			out.println("}");
		} catch (JWriterDirectorException e) {
			throw new CodeWriterException("Troubles with java writer director.",e);
		} catch (java.rmi.RemoteException e) {
			throw new CodeWriterException("Can't access TIR.",e);
		} catch (SOFA.SOFAnode.Made.TIR.TIRExceptLock e) {
			throw new CodeWriterException("TIR is locked.",e);
		}

		out.close();
	}
}
