/*
 * EEGenerator.java
 *
 * Created on 2. duben 2002, 12:10
 */

package SOFA.Connector.EEG;

import org.w3c.dom.*;

/**
 *
 * @author  ghort
 * @version 
 */
public class EEGenerator {

	protected java.util.LinkedList plugin; /* of EEGPluginInterface */
	
	protected javax.xml.parsers.DocumentBuilderFactory documentBuilderFactory;
	protected javax.xml.parsers.DocumentBuilder documentBuilder;
	
	protected Document configDoc;

	static public String getTypesPackage() {
		return "SOFA.Connector.EEG.Types";
	}
	
	/** Creates new EEGenerator */
	public EEGenerator() throws EEGeneratorException {
		try {
			documentBuilderFactory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
			documentBuilder=documentBuilderFactory.newDocumentBuilder();
		
			configDoc=documentBuilder.parse(new java.io.File(getEEGConfigRoot()+"/"+System.getProperty("SOFA.Connector.EEG.Config","conf.xml")));
			
			Element elParent=configDoc.getDocumentElement();
			Node elPlugins, elPlugin, elProp;
			
			plugin=new java.util.LinkedList();
			
			System.out.println("Loading EEG plugins:");
			elPlugins=elParent.getFirstChild();
			while (elPlugins!=null) {
				if (elPlugins.getNodeType()==Node.ELEMENT_NODE && elPlugins.getNodeName().equals("EEGPlugins")) {
					elPlugin=elPlugins.getFirstChild();
					while (elPlugin!=null) {
						if (elPlugin.getNodeType()==Node.ELEMENT_NODE && elPlugin.getNodeName().equals("EEGPlugin")) {
							
							java.util.HashMap props=new java.util.HashMap();
							elProp=elPlugin.getFirstChild();
							while (elProp!=null) {
								if (elProp.getNodeType()==Node.ELEMENT_NODE) {
									Element el=(Element)elProp;
									if (el.getNodeName().equals("property") && el.hasAttribute("name") && el.hasAttribute("value")) {
										props.put(el.getAttribute("name"),el.getAttribute("value"));
									}
								}
								elProp=elProp.getNextSibling();
							}
							
							System.out.println("  Loading plugin ("+((Element)elPlugin).getAttribute("class")+"):");
									
							Class cls=Class.forName(((Element)elPlugin).getAttribute("class"));
							Class[] coPar=new Class[1];
							coPar[0]=Class.forName("java.util.HashMap");
							java.lang.reflect.Constructor co=cls.getConstructor(coPar);
							
							Object[] coInst=new Object[1];
							coInst[0]=props;
							EEGPluginInterface plgObject=(EEGPluginInterface)co.newInstance(coInst);
							
							plugin.add(plgObject);
							System.out.println("  Plugin "+plgObject.getName()+" ("+((Element)elPlugin).getAttribute("class")+") loaded.");
						}
						elPlugin=elPlugin.getNextSibling();
					}
				}
				elPlugins=elPlugins.getNextSibling();
			}
						
		} catch (Exception e) {
			throw new EEGeneratorException("Can't parse configuration",e);
		}
	}

	public ElementsTechnologyDescriptor canGenerate(ElementQuery query) {
		java.util.Iterator iter;
		EEGPluginInterface plgObject;
		ElementsTechnologyDescriptor etDescr=new ElementsTechnologyDescriptor();
		java.util.LinkedList tDescr=new java.util.LinkedList();
		
		etDescr.type=query.type;
		etDescr.instanceName=query.instanceName;
		iter=plugin.iterator();
		try {
			for (;;) {
				plgObject=(EEGPluginInterface)iter.next();
				tDescr.addAll(java.util.Arrays.asList(plgObject.canGenerate(query)));
			}
		} catch (java.util.NoSuchElementException e) {
		}
		
		int i;
		Object arr[];
		etDescr.technologies=new ElementTechnologyDescriptor[(arr=tDescr.toArray()).length];
		for (i=0;i<arr.length;i++) {
			etDescr.technologies[i]=(ElementTechnologyDescriptor)arr[i];
		}

		return etDescr;
	}
	
	public ElementOutputDescriptor generate(ElementInputDescriptor elDescr) throws EEGeneratorException {
		java.util.Iterator iter;
		EEGPluginInterface plgObject;
		
		int colonIndex=elDescr.implementation.indexOf(':');
		String pluginName;
		
		if (colonIndex==-1) {
			pluginName=elDescr.implementation;
		} else {
			pluginName=elDescr.implementation.substring(0,colonIndex);
		}
		
		System.out.println("  Generating element '"+elDescr.implementation+"'");
		
		iter=plugin.iterator();
		try {
			for (;;) {
				plgObject=(EEGPluginInterface)iter.next();
				if (pluginName.equals(plgObject.getName())) {
					try {
						ElementOutputDescriptor ret;
						ret=plgObject.generate(elDescr);
						return ret;
					} catch (EEGPluginException e) {
						throw new EEGeneratorException("Plugin failed",e);
					}
				}
			}
		} catch (java.util.NoSuchElementException e) {
			throw new EEGeneratorException("Can't generate element.",e);
		}
	}

	static public String getEEGRoot() throws EEGeneratorException {
		try {
			return System.getProperty("SOFA.Connector.Root","SOFA/Connector")+"/EEG";
		} catch (Exception e) {
			throw new EEGeneratorException("Can't access property SOFA.Connector.Root",e);
		}
	}

	static public String getEEGConfigRoot() throws EEGeneratorException {
		String configDir;
		try {
			configDir=System.getProperty("SOFA.Connector.Config");
		} catch (Exception e) {
			throw new EEGeneratorException("Can't access property SOFA.Connector.Config",e);
		}
		if (configDir==null) {
			configDir=getEEGRoot();
		} else {
			configDir+="/EEG";
		}
		return configDir;
	}
}
