/* $Id: ExprConstantImpl.java,v 2.0 2002/02/15 13:55:29 hnetynka Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.IOException;

public class ExprConstantImpl extends UnicastRemoteObject implements ExprConstant, TIRImplObject {
  ConstantDef cnst;
  CDLType loadCnst;
  ExprOperKindImpl eok;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl; 

  public ExprConstantImpl(ConstantDef constant) throws RemoteException {
    cnst = constant;
    loadCnst = null;
    eok = new ExprOperKindImpl(ExprOperKind.eok_constant);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
  }

  public ExprConstantImpl()  throws RemoteException {
    cnst = null;
    loadCnst = null;
    eok = new ExprOperKindImpl(ExprOperKind.eok_constant);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public ConstantDef ref_const() throws RemoteException {
    return cnst;
  }

  public ExprOperKind get_eok_kind() throws RemoteException {
    return (ExprOperKind) eok;
  }
  
  public ExprOperDef calculate() throws RemoteException, TIRExceptCalculate {
    return cnst.value().calculate();
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile.writeInt(eok.value());
      Identification idl = ((Contained)cnst).get_identification();
      Storage.writeString(st.curOutFile, idl.lang_absolute_name().name());
      Storage.writeString(st.curOutFile, idl.name());
      Storage.writeString(st.curOutFile, idl.version());
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in directory "+st.current+".");
    }
  }
  
  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      String absn = Storage.readString(st.curInFile);
      String name = Storage.readString(st.curInFile);
      String ver = Storage.readString(st.curInFile);
      loadCnst = new NotYetInRep(absn,name,ver);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in directory "+st.current+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    try {
    
    if ((loadCnst!=null) && (loadCnst.get_def_kind().value() == DefinitionKind.dk_none)) {   // is tp NotYetInRep?
      AbsoluteName nm = ((NotYetInRep) loadCnst).nm;
      Container con = r;// begin search from repository
      Contained[] rt;
      Contained ret;
      IdentificationImpl id;
      for (int i=0;i<nm.size()-1;i++) {  // last element is name of constant
	id = new IdentificationImpl(nm.elementAt(0),"::"+nm.elementAt(i),"");
        rt = ((SContainer)con).slookup_name(id);
	if (rt.length==0) throw new TIRExceptStorage("Can't find type " + nm.name());
        if (rt.length==1) {
          con = (Container) rt[0];
	} else {  // const in iface (etc.) - more versions of this iface
	  boolean found = false;
          id.version(((NotYetInRep) loadCnst).id.version());
          int j = 0;
	  for (j=0;j<rt.length;j++) {
	    if (id.is_short_equal(rt[j].get_identification())) {
	      found = true;
              break;  // -- break for
	    }
	  } // -- end of for
	  if (!found) throw new TIRExceptStorage("Can't find type " + nm.name());
	  con = (Container) rt[j];
	}
      }
      if ((ret = ((SContainer)con).slookup(((NotYetInRep) loadCnst).id))==null)
      throw new TIRExceptStorage("Can't find type " + nm.name());
      cnst = (ConstantDef) ret;
      loadCnst = null;
    }

    } catch (TIRExceptLock e) {;}
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_work) {
      stKindImpl.toNormal();
      Identification idl = cnst.get_identification();
      loadCnst = new NotYetInRep(idl.lang_absolute_name().name(), idl.name(), idl.version());
      cnst = null;
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
