/* $Id: Storage.java,v 2.1 2002/04/19 12:25:22 hnetynka Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.io.*;
import java.rmi.RemoteException;

class Storage {
  public static final String indexFile = "index.tir";
  public static final int maxLenFileName = 8;
  public File root;      /* root directory of storage */
  public File current;   /* current directory of storage */
  public File currentFile;
  public DataInputStream curInFile;
  public DataOutputStream curOutFile;
  
  public Storage(File r, File cur) {
    root = r;
    current = cur;
    currentFile = null;
    curInFile = null;
    curOutFile = null;
  }

  /* read childs from disk (== read info from index.tir) 
     ret[][0] == file(dir)name
     ret[][1] == defKind
     ret[][2] == name
     ret[][3] == version
  */   
  public String[][] listOfChilds() throws TIRExceptStorage, RemoteException  {
    try {
      File f = new File(current,indexFile);
      BufferedReader in = new BufferedReader(new FileReader(f));
      boolean done = false;
      String str;
      int num = 0;
      boolean b = false;
      while (!done) {
        str = in.readLine();
	if (str==null)
	  done = true;
	else {
          if (b)
	    num++;
	  else  
	    if (str.compareTo("CHILDS")==0)
	      b = true;
	}      
      }
      in.close();
      String[][] ret = new String [num][4];
      
      if (num==0)
        return ret;
      in = new BufferedReader(new FileReader(f));
      b = false;
      done = false;
      while (!done) {
        str = in.readLine();
	if (str.compareTo("CHILDS")==0)
	  done = true;
      }
      int j,k;
      for (int i=0;i<num;i++) {
        k = 0;
        str = in.readLine();
	StringBuffer sb = new StringBuffer();
	for(j=0;j<str.length();j++) {
          if (k<3 && str.charAt(j)=='?') {
	    ret[i][k] = sb.toString();
            k++;
	    sb = new StringBuffer();
	  } else {
            sb.append(str.charAt(j));
	  }
	}
	ret[i][k] = sb.toString();
      }
      in.close();
      return ret;
    } catch (IOException e2) {
      throw new TIRExceptStorage("Can't access storage at \""+current.getAbsolutePath()+"\"");
    }
  } 
/*
  public static void main(String[] argv) throws TIRExceptStorage, RemoteException{
    File f = new File("/home/petr/tmp");
    Storage st = new Storage(f,f);
    String[][] s = st.listOfChilds();
    for(int i=0;i<s.length;i++) {
      System.out.println(s[i][0]+"?"+s[i][1]+"?"+s[i][2]+"?"+s[i][3]);
    }
  } */

  /* Search identification in array returned by listOfChilds() */
  public String[] existsChild(String[][] s, IdentificationImpl id)  throws RemoteException{
    if (s==null)
      return null;
    for(int i=0;i<s.length;i++) {
      if (id.is_short_equal(s[i][2],s[i][3]))
        return s[i];
    }
    return null;
  }

  /* read string from stream */
  public static String readString(DataInputStream in) throws IOException {
    StringBuffer sb = new StringBuffer();
    int len;
    len = in.readInt();
    for(int i=0;i<len;i++) {
      sb.append(in.readChar());
    }
    return sb.toString();
  }

  /* write string to stream */
  public static void writeString(DataOutputStream out, String str) throws IOException {
    out.writeInt(str.length());
    out.writeChars(str);
  }

  public static void writeExprOperDef(Storage st, ExprOperDef ex) throws TIRExceptStorage, RemoteException {
    ((TIRImplObject) ex).save(st);
  }
  
  public static ExprOperDef readExprOperDef(Storage st) throws TIRExceptStorage {
    ExprOperDef ret = null;
    try {
      int kind = st.curInFile.readInt();
      switch (kind) {
        case ExprOperKind.eok_short:
	  ret = new ExprShortImpl();
	  break;
	case ExprOperKind.eok_long:
	  ret = new ExprLongImpl();
	  break;
	case ExprOperKind.eok_longlong:
	  ret = new ExprLongLongImpl();
	  break;
        case ExprOperKind.eok_unsigshort:
	  ret = new ExprUnsigShortImpl();
	  break;
	case ExprOperKind.eok_unsiglong:
	  ret = new ExprUnsigLongImpl();
	  break;
	case ExprOperKind.eok_unsiglonglong:
	  ret = new ExprUnsigLongLongImpl();
	  break;
        case ExprOperKind.eok_float:
	  ret = new ExprFloatImpl();
	  break;
        case ExprOperKind.eok_double:
	  ret = new ExprDoubleImpl();
	  break;
        case ExprOperKind.eok_fixed:
	  ret = new ExprFixedImpl();
	  break;
        case ExprOperKind.eok_octet:
	  ret = new ExprOctetImpl();
	  break;
        case ExprOperKind.eok_default:
          ret = new ExprDefaultImpl(StateKind.sk_normal);
          break;
        case ExprOperKind.eok_boolean:
	  ret = new ExprBooleanImpl();
	  break;
        case ExprOperKind.eok_char:
	  ret = new ExprCharImpl();
	  break;
        case ExprOperKind.eok_wchar:
	  ret = new ExprWcharImpl();
	  break;
        case ExprOperKind.eok_string:
	  ret = new ExprStringImpl();
	  break;
        case ExprOperKind.eok_wstring:
	  ret = new ExprWstringImpl();
	  break;
        case ExprOperKind.eok_valuegen:
	  ret = new ValueGenImpl(StateKind.sk_normal);
	  break;
        case ExprOperKind.eok_property:
	  ret = new ExprPropertyImpl();
	  break;
        case ExprOperKind.eok_constant:
	  ret = new ExprConstantImpl();
	  break;
        case ExprOperKind.eok_enum:
	  ret = new ExprEnumImpl();
	  break;
        case ExprOperKind.eok_or:
        case ExprOperKind.eok_xor:
        case ExprOperKind.eok_and:
        case ExprOperKind.eok_shr:
        case ExprOperKind.eok_shl:
        case ExprOperKind.eok_plus:
        case ExprOperKind.eok_minus:
        case ExprOperKind.eok_mul:
        case ExprOperKind.eok_div:
        case ExprOperKind.eok_mod:
	  ret = new ExprBinOperationDefImpl(kind);
	  break;
	case ExprOperKind.eok_unplus:
	case ExprOperKind.eok_unminus:
	case ExprOperKind.eok_untilde:
	  ret = new ExprUnOperationDefImpl(kind);
	  break;
	/* dopsat */
	default:
	  throw new TIRExceptStorage("Unexpected kind ("+kind+")of object in \""
	                              +st.currentFile+"\".");
      }
      ((TIRImplObject) ret).load(st);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in directory "+st.current+".");
    }
    return ret;
  }

  public static void writeCDLType(Storage st, CDLType type) throws TIRExceptStorage, IOException, RemoteException {
    int dkind = type.get_def_kind().value();
    st.curOutFile.writeInt(dkind);   // saving kind 
    switch (dkind) {
      case DefinitionKind.dk_Primitive:
        st.curOutFile.writeInt(((PrimitiveDef)type).kind().value());
        break;
      case DefinitionKind.dk_Array:
      case DefinitionKind.dk_Sequence:
        ((TIRImplObject)type).save(st);
        break;
      case DefinitionKind.dk_String:
        ((TIRImplObject)((StringDef)type).bound()).save(st);
        break;
      case DefinitionKind.dk_Wstring:
        ((TIRImplObject)((WstringDef)type).bound()).save(st);
        break;
      case DefinitionKind.dk_Fixed:
        ((TIRImplObject)type).save(st);
	break;
      case DefinitionKind.dk_Struct:
      case DefinitionKind.dk_Union:
      case DefinitionKind.dk_Enum:
      case DefinitionKind.dk_Typedef:
      case DefinitionKind.dk_Interface:
      case DefinitionKind.dk_Exception:
      case DefinitionKind.dk_Frame:
	Identification idl = ((Contained)type).get_identification();
        Storage.writeString(st.curOutFile, idl.lang_absolute_name().name());
	Storage.writeString(st.curOutFile, idl.name());
	Storage.writeString(st.curOutFile, idl.version());
	break;
    }
  }

  public static CDLType readCDLType(Storage st) throws TIRExceptStorage, IOException, RemoteException {
    int dkind = st.curInFile.readInt();
    int a;
    CDLType ret = null;
    ExprOperDef expr;
    switch (dkind) {
      case DefinitionKind.dk_Primitive:
        a = st.curInFile.readInt();
	ret = new NotYetInRep(a);
	break;
      case DefinitionKind.dk_Array:
        ret = new ArrayDefImpl();
	((TIRImplObject) ret).load(st);
	break;
      case DefinitionKind.dk_Sequence:
        ret = new SequenceDefImpl();
	((TIRImplObject) ret).load(st);
	break;
      case DefinitionKind.dk_String:
        expr = Storage.readExprOperDef(st);
        ret = new StringDefImpl(expr);
	break;
      case DefinitionKind.dk_Wstring:
        expr = Storage.readExprOperDef(st);
        ret = new WstringDefImpl(expr);
	break;
      case DefinitionKind.dk_Fixed:
        ret = new FixedDefImpl();
        ((TIRImplObject) ret).load(st);
	break;
      case DefinitionKind.dk_Struct:
      case DefinitionKind.dk_Union:
      case DefinitionKind.dk_Enum:
      case DefinitionKind.dk_Typedef:
      case DefinitionKind.dk_Interface:
      case DefinitionKind.dk_Exception:
      case DefinitionKind.dk_Frame:
	String absn = readString(st.curInFile);
	String name = readString(st.curInFile);
	String ver = readString(st.curInFile);
	ret = new NotYetInRep(absn,name,ver);
	break;
      default:
        throw new TIRExceptStorage("Unexpected kind ("+dkind+") of type in \""
	                              +st.currentFile+"\".");
    }
    
    return ret;
  }

  public static ProtocolOperDef readProtocolOperDef(Storage st) throws TIRExceptStorage {
    ProtocolOperDef ret = null;
    try {
      int kind = st.curInFile.readInt();
      switch (kind) {
        case ProtocolOperKind.pok_operand:
	  ret = new ProtocolOperandDefImpl();
	  break;
        case ProtocolOperKind.pok_nulloperand:
	  ret = new ProtocolNullOperandDefImpl(StateKind.sk_normal);
	  break;
        case ProtocolOperKind.pok_metaoperand:
	  ret = new ProtocolMetaOperandDefImpl();
	  break;
        case ProtocolOperKind.pok_nestedoperand:
	  ret = new ProtocolNestedOperandDefImpl();
	  break;
        case ProtocolOperKind.pok_repetition:
	  ret = new ProtocolUnOperationDefImpl(kind);
	  break;
        case ProtocolOperKind.pok_combination:
        case ProtocolOperKind.pok_andparallel:
        case ProtocolOperKind.pok_orparallel:
        case ProtocolOperKind.pok_sequence:
        case ProtocolOperKind.pok_alternative:
	case ProtocolOperKind.pok_restriction:
	  ret = new ProtocolBinOperationDefImpl(kind);
	  break;
	default:
	  throw new TIRExceptStorage("Unexpected kind (ProtocolKind "+kind+")of object in \""
	                              +st.currentFile+"\".");
      }
      ((TIRImplObject) ret).load(st);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in directory "+st.current+".");
    }
    return ret;
  }

  public static void writeProtocolOperDef(Storage st, ProtocolOperDef pr) throws TIRExceptStorage, RemoteException {
    ((TIRImplObject) pr).save(st);
  }
  
}


