/* $Id: Repository.java,v 2.0 2002/02/15 13:55:14 hnetynka Exp $ */
package SOFA.SOFAnode.Made.TIR;
import java.rmi.RemoteException;

/** Root of the <b>TIR</b> objects. */
public interface Repository extends Container {
  /** Returns your own copy of repository. You can call create_* methods on this work
      repository. */
  public WorkRepository beginChanges(ProfileDef p) throws RemoteException, TIRExceptCreate, TIRExceptLock; 
  /** Returns object for management of repository. */
  public ManageRepository getManage() throws RemoteException;
  /** Returns object for management of profiles. */
  public ManageProfiles getProfiles() throws RemoteException;

  /** get DefinitionKind object with specified kind. For serching purposes. */
  public DefinitionKind get_spec_def_kind(int kind) throws RemoteException;

  /** Create new identification with given language, name and version. */
  public Identification create_identification(String lang, String name, String version) throws RemoteException, TIRExceptCreate;
  /** Create new idetntification with next version from the given id. */
  public Identification create_nextversion(Identification id) throws RemoteException, TIRExceptCreate;
  /** Create new idetntification with initial version. */
  public Identification create_initialversion(String lang, String name) throws RemoteException, TIRExceptCreate;
  /** Create new identification in the new branch from the given id. */
  public Identification create_branch(Identification from, String branchTag) throws RemoteException, TIRExceptCreate;

  // next methods create elements of the expressions 
  public ExprBoolean    create_exprboolean(boolean val) throws RemoteException, TIRExceptCreate;
  public ExprShort    create_exprshort(short val) throws RemoteException, TIRExceptCreate;
  public ExprOctet    create_exproctet(byte val) throws RemoteException, TIRExceptCreate;
  public ExprLong     create_exprlong(int val) throws RemoteException, TIRExceptCreate;
  public ExprLongLong create_exprlonglong(long val) throws RemoteException, TIRExceptCreate;
  public ExprUnsigShort create_exprunsigshort(short val) throws RemoteException, TIRExceptCreate;
  public ExprUnsigLong  create_exprunsiglong(int val) throws RemoteException, TIRExceptCreate;
  public ExprUnsigLongLong create_exprunsiglonglong(long val) throws RemoteException, TIRExceptCreate;
  public ExprFloat    create_exprfloat(float val) throws RemoteException, TIRExceptCreate;
  public ExprDouble   create_exprdouble(double val) throws RemoteException, TIRExceptCreate;
  public ExprLongDouble create_exprlongdouble(double val) throws RemoteException, TIRExceptCreate;
  public ExprFixed    create_exprfixed(java.math.BigDecimal val, int digits, int scale) throws RemoteException, TIRExceptCreate;
  public ExprProperty create_exprproperty(String pr) throws RemoteException, TIRExceptCreate;
  public ExprEnum create_exprenum(EnumDef en, String enMember) throws RemoteException, TIRExceptCreate, TIRExceptLock;
  public ExprDefault  create_exprdefault() throws  RemoteException, TIRExceptCreate;
  public ExprConstant create_exprconstant(ConstantDef val) throws RemoteException, TIRExceptCreate;
  public ExprChar    create_exprchar(char val) throws RemoteException, TIRExceptCreate;
  public ExprWchar   create_exprwchar(char val) throws RemoteException, TIRExceptCreate;
  public ExprString  create_exprstring(String val) throws RemoteException, TIRExceptCreate;
  public ExprWstring create_exprwstring(String val) throws RemoteException, TIRExceptCreate;
  public ExprUnOperationDef create_exprunoperation(int kind, ExprOperDef op) throws RemoteException, TIRExceptCreate;
  public ExprBinOperationDef create_exprbinoperation(int kind, ExprOperDef op1, ExprOperDef op2) throws RemoteException, TIRExceptCreate;
  public ValueGen create_valuegen() throws RemoteException, TIRExceptCreate;
}
