/* $Id: DeplDockRegistryServer.java,v 1.2 2002/10/08 16:14:15 hnetynka Exp $ */
package SOFA.SOFAnode.Run.DeplDockRegistry;

import SOFA.SOFAnode.Run.DeplDockRegistry.*;

import java.util.Hashtable;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POA;

import java.net.InetAddress;
import java.net.UnknownHostException;

/** Deployment dock registry servant
  * 
  * @author Petr Hnetynka
  */
class DeplDockRegistryServant implements DeplDockRegistryOperations {
  
  Hashtable store;
  long id;
  String nodeName;
  SOFA.Util.Lock lock;
  
  public DeplDockRegistryServant (String nodeName) {
    store = new Hashtable();
    id = 0;
    this.nodeName = nodeName;
    lock = new SOFA.Util.Lock(false);
  }
  
  public short register (byte[] sofaReference, String name) {
    if (store.containsKey(name)) {
      return 1;
    }
    store.put(new String(name), new Item(sofaReference));
    return 0;
  }
  
  public byte[] lookup (String name) throws SOFA.SOFAnode.Run.DeplDockRegistry.NotFoundException {
    Item a = (Item) store.get(name);
    if (a == null)
      throw new NotFoundException();
    return a.obj;
  }

  public String[] getAllNames () {
    String[] ret = new String [store.size()] ;
    int i = 0;
    for (java.util.Enumeration e = store.keys() ; e.hasMoreElements() ;) {
      ret[i] = new String ((String) e.nextElement());
      i++;
    }
    return ret;
  }

  public String  getUniqueID() {
    String ret;
    lock.lock();
    ret = nodeName + Long.toString(id++);
    lock.unlock();
    return ret;
  }
}

class Item {
  byte[] obj;
  public Item(byte[] o) { obj = o; }
}

/** Implementation.
  * 
  * @author Petr Hnetynka
  */
public class DeplDockRegistryServer {
  public static void main(String[] argv) {
    try{
      String sofaNodeName = System.getProperty("sofa.nodename","");
      
      if (sofaNodeName.compareTo("")==0) {
        try {
          sofaNodeName = InetAddress.getLocalHost().getHostName();
        } catch (UnknownHostException e) { 
          System.err.println("Can't get SOFAnode name. Set name manually.");
          System.exit(1);
        }
      }

/*      for (int i=0; i<argv.length; i++) {
        System.out.println(argv[i]);
      }*/
     
      ORB orb = ORB.init(argv, null);
 
      DeplDockRegistryServant servant = new DeplDockRegistryServant(sofaNodeName);

      POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
      rootpoa.the_POAManager().activate();
     
      DeplDockRegistryPOATie tie = new DeplDockRegistryPOATie(servant, rootpoa);
      DeplDockRegistry ref = tie._this(orb);
 
      org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
      NamingContext ncRef = NamingContextHelper.narrow(objRef);
 
      NameComponent nc = new NameComponent("DeplDockRegistry", "");
      NameComponent path[] = { nc };
      ncRef.rebind(path, ref);

      System.out.println("DeplDockRegistry is running.");

      java.lang.Object sync = new java.lang.Object();
      synchronized (sync) {
        sync.wait();
      }
 
    } catch (Exception e) {
      System.err.println("Exception: " + e);
      e.printStackTrace(System.out);
    } 
  }
}
