/* $Id: SOFAInterfaceClassLoader.java,v 1.4 2003/01/06 11:04:50 hnetynka Exp $ */
package SOFA.SOFAnode.Run;

import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

/** SOFA interface loader. It loads interfaces (all cdl defined types) of applications
  * and also it loads connectors.
  *
  * @author Petr Hnetynka
  */
public class SOFAInterfaceClassLoader extends ClassLoader {
  /** URL with classes */
  URL url;
  /** Names of CDL defined entities and their equivalents in Java */
  java.util.Hashtable  cdlEntities;
  java.util.Properties trIndex;

  /** Constructor.
    * @param _url      the URL from which to load classes
    * @param _cdlEntities  cdl entities defined in depldescriptor
    */
  public SOFAInterfaceClassLoader(URL _url, java.util.Hashtable _cdlEntities) {
    super();
    url = _url;
    cdlEntities = _cdlEntities;
    
    URL defURL;
    try {
      defURL = new URL(url.toString()+"iface/index");
    } catch (MalformedURLException e) {
      throw new SOFARuntimeException("MalformedURLException: "+e.getMessage(), e);
    }
    try {
      trIndex = new java.util.Properties();
      URLConnection con = defURL.openConnection();
      InputStream is = con.getInputStream();
      trIndex.load(is);
      is.close();
    } catch (IOException e) {
      throw new SOFARuntimeException("IOException: "+e.getMessage(), e);
    }
  }
  
  public Class findClass(String name) throws ClassNotFoundException {
    byte[] b;
//    System.out.println("???: "+name);
    b = loadClassData(name);
    Class ret = defineClass(name, b, 0, b.length);
//    System.out.println("??? End: "+name);
    return ret;
  }

  private byte[] loadClassData(String name) throws ClassNotFoundException {
//    System.out.println("loadClassData name = "+name);

    java.util.StringTokenizer tok = new java.util.StringTokenizer(name, ".");
    StringBuffer path = new StringBuffer();
    String nm = "";
    while (tok.hasMoreTokens()) {
      nm = tok.nextToken();
      path.append("/");
      path.append(nm);
    }
    String conPath = url.toString()+"conn"+path.toString()+".class";

    String cdlName = (String) cdlEntities.get(name);
    if (cdlName == null) {
      URL defURL;
      try {
        defURL = new URL(conPath);
      } catch (MalformedURLException e2) {
        throw new ClassNotFoundException("MalformedURLException: "+e2.getMessage());
      }
      try {
        URLConnection con = defURL.openConnection();
        InputStream is = con.getInputStream();
        ByteArrayOutputStream bo = new ByteArrayOutputStream();
        int a = is.read();
        while (a != -1) {
          bo.write(a);
          a = is.read();
        }
        is.close();
        byte[] ar = bo.toByteArray();
        bo.close();
        return ar;
      } catch (IOException e2) {
        throw new ClassNotFoundException("IOException: "+e2.getMessage(), e2);
      }
    }
    
    String subdir = (String) trIndex.get(cdlName);
    if (subdir == null)
      throw new ClassNotFoundException("No such type in TR");


    String iPath = url.toString()+"iface/"+subdir+path.toString()+".class";

    URL defURL;
    try {
      defURL = new URL(iPath);
    } catch (MalformedURLException e) {
      throw new ClassNotFoundException("MalformedURLException: "+e.getMessage());
    }
    try {
      URLConnection con = defURL.openConnection();
      InputStream is = con.getInputStream();
      ByteArrayOutputStream bo = new ByteArrayOutputStream();
      int a = is.read();
      while (a != -1) {
        bo.write(a);
        a = is.read();
      }
      is.close();
      byte[] ar = bo.toByteArray();
      bo.close();
      return ar;
    } catch (IOException e) {
      try {
        defURL = new URL(conPath);
      } catch (MalformedURLException e2) {
        throw new ClassNotFoundException("MalformedURLException: "+e2.getMessage());
      }
      try {
        URLConnection con = defURL.openConnection();
        InputStream is = con.getInputStream();
        ByteArrayOutputStream bo = new ByteArrayOutputStream();
        int a = is.read();
        while (a != -1) {
          bo.write(a);
          a = is.read();
        }
        is.close();
        byte[] ar = bo.toByteArray();
        bo.close();
        return ar;
      } catch (IOException e2) {
        throw new ClassNotFoundException("IOException: "+e2.getMessage());
      }
    }
  }

  /** Returns sofa interface class loader.
    * Base url (from which to load classes) is specified by java property <tt>sofa.tr.url</tt>
    * @param cdlEntities   hashtable with cdl entities (based on depldescr)
    */
  public static ClassLoader getInterfaceClassLoader(java.util.Hashtable cdlEntities) throws SOFARuntimeException {
    try {
      String baseURL = System.getProperty("sofa.tr.url","file:/sofa/tr/");
      return new SOFAInterfaceClassLoader( new URL(baseURL), cdlEntities);
    } catch (MalformedURLException e) {
      throw new SOFARuntimeException("MalformedURLException: "+e.getMessage());
    }
  }
}
