/* $Id: Lock.java,v 1.1 2002/06/24 22:24:40 hnetynka Exp $ */
package SOFA.Util;

/** Lock.
  *
  * @author Petr Hnetynka
  */
public class Lock {

  private boolean locked;

  /** Constructor.
    * @param l  if should be locked or not
    */
  public Lock(boolean l) {
    locked = l;
  }

  /** Lock. If lock is locked, then wait.
    * @return false, if waiting was interrupted
    */
  synchronized public boolean lock() {
    try {
      while (locked) 
        wait();         /* wait until somenone call notify() */
      locked = true;
    } catch (InterruptedException e) {
      return false;
    }
    return true;
  }

  /** Try lock.
    * @return false, if can't get lock immediatelly
    */
  synchronized public boolean tryLock() {
    if (locked) 
      return false;
    else {
      locked = true;
      return true;
    }
  }
  
  /** Unlock.
    */
  synchronized public void unlock() {
    locked = false;
    notify();  /* wake up others which wait for lock */
  }
  
  /** Test if lock is locked */
  synchronized public boolean isLocked() {
    return locked;
  }
}
